begin
  # Try to require from the libdir
  require 'puppetpe/puppetlabs/pe_console/console_http'
rescue LoadError
  # Fallback to loading relatively from the modulepath
  require File.join(File.dirname(__FILE__), '../../..', 'puppetpe/puppetlabs/pe_console/console_http')
end

class Puppet::Node::Console < Puppet::Indirector::Code
  include PuppetPE::Puppetlabs::PEConsole::ConsoleHTTP

  def initialize
    super
    load_config
  end

  def find(request)
    response = do_request do |connection, headers|
      connection.get("/nodes/#{request.key}", headers)
    end

    # Translate the output to ruby.
    result = translate(request.key, response)

    # Set the requested environment if it wasn't overridden
    # If we don't do this it gets set to the local default
    result[:environment] ||= request.environment.name

    create_node(request.key, result)
  end

  private

  ## Classification helpers ##

  # Turn our outputted objects into a Puppet::Node instance
  def create_node(name, result)
    node = Puppet::Node.new(name)

    [:parameters, :classes, :environment].each do |param|
      if value = result[param]
        node.send(param.to_s + "=", value)
      end
    end

    node.fact_merge
    node
  end

  # Translate the yaml string into Ruby objects
  def translate(name, output)
    YAML.load(output).inject({}) do |hash, data|
      case data[0]
      when String
        hash[data[0].intern] = data[1]
      when Symbol
        hash[data[0]] = data[1]
      else
        raise Puppet::Error, "key is a #{data[0].class}, not a string or symbol"
      end

      hash
    end

  rescue => detail
      raise Puppet::Error, "Could not load node classification for #{name}: #{detail}"
  end

end
